function [bb,energy_spent,Eopt,Tn]=balance_b(b,E0,E_S,omega,C,D,T,R,s)

%IMPORTANT: In C and D, the lower the index the higher the battery level
%For instance: C(1) is a power parameter of a node with 90% battery
%              C(7) is a power parameter of a node with 15% battery
%********************INPUTS******************************************
%b is the optimal ceiled modulation parameters
%E0 is the theoretical optimal spending energy in each node
%E_S is the energy ratio spent respect the theoric optimal
%omega is the amount of energy that we want to save in node 1
% omega  [0,1]

%C is a vector(row) that symbolizes the tx power
%D is a vector(row) that symbolizes the electronic power
%R is the transmission rate
%s is the packet size
%T total path latency
%***********************OUTPUTS**************************************
%bb (vector) is the optimal modulation parameter ceiled
%energy_spent is the energy ratio spent respect the theoretical optimal
%Eopt (vector) are the optimal energies for balanced nodes with ceiling
%Tn are the tx times for balanced nodes with ceiling
%********************************************************************


%disp('####################################################################')
%disp('####################################################################')
disp('computing balance...')

%bmax is the limit considered for not falling in an infinite bucle
%when performing T check.
bmax=10;
err=0;
energy_spent=E_S;



%CHECKING OMEGA(energy saved in node 1)
disp('checking omega...')
while (1-energy_spent(length(C))) < (omega*(0.001+1))
    b(length(C))=b(length(C))-1;
%    disp('decreasin modulation parameter of target node')
    
    if b(length(C))==1
        disp('cant reach energy savings in target node ...')
        err=-1;
        break
    end
    
    Eopt(length(C))=((C(length(C)).*((2.^b(length(C)))-1)+(D(length(C)))).*s./b(length(C)));
    energy_spent(length(C))=Eopt(length(C))./E0;
end


if err==-1
    bb=-1;
    energy_spent=-1;
    Eopt=-1;
    Tn=-1;
    return
end





%CHECKING T
disp('checking T...')
Tn=s./(b.*R);
Ts=sum(Tn);
%j=length(C); this is wrong
j=1;
while Ts > (T*(0.001+1))
    b(j)=b(j)+1;
%    fprintf('increasing modulation parameter of node %d \n',j)
    Tn(j)=s./(b(j).*R);
    Ts=sum(Tn);
    
    if min(b(1:(length(b)-1)))>=bmax
        if Ts > (T*(0.001+1))
%            disp('scaping from infinite bucle...')
            %fprintf('energy savings in node 1 cant be reached with %d balancing node/s \n',(length(C)-1))
            err=-1;
            break
        end
    end
    
  
    %j=j-1; wrong
    j=j+1;
    if j==length(b)
        j=1;
    end
    
    %if j==1    wrong
    %    j=length(C);
    %end
end

bb=b;

if err==-1
    bb=b.*Inf;
    energy_spent=E_S.*Inf;
    Eopt=bb;
    Tn=bb;
    return
end


%ENERGY SPENDING RESULTS
for i = 1:length(C)
    Eopt(i)=((C(i).*((2.^b(i))-1)+(D(i))).*s./b(i));
end
energy_spent=Eopt./E0;

%disp('balance of b`s finished')
%disp('####################################################################')
%disp('####################################################################')


%End of balance_b