function [node,groups_balanced,not_balanced]=main(C,D,Lat,R,s,node_energy)
%------------------INPUTS----------------------
%C is a vector(row) that symbolizes the tx power
%D is a vector(row) that symbolizes the electronic power
%R is the transmission rate
%s is the packet size
%Lat total path latency
%node_energy is the energy of each node 
%------------------OUTPUTS---------------------
%groups_balanced is a vector that contains the positions of a 
%balanced node group in the vector node which has been put in 
%order from lower batt til higher batt
%not_balanced are the nodes not balanced due to several reasons
%----------------------------------------------
                
max_energy=10; %in Jules
max_group=5;

%structure of node
node=[];
node.index=0;
node.C=0;           %tx power parameter
node.D=0;           %electronic parameter
node.b=0;           %mod parameter balanced
node.b0=0;          %theoretical optimal mod param
node.bop=0;         %theoretical optimal ceiled mod param
node.T=0;           %latency of node i of the path
node.energy=0;      %remaining battery level
node.Eused=0;       %energy used
node.ratio=0;
node.omega=0;    %energy that we want to save if its target node
node.gamma=0;    %max over-power-spending factor if its balancing node 


%compute the optimal Energies and tx times for each node if we consider
%the whole path 
[E0,T0]=optimal_energy(Lat,length(C),C,D,R,s,0.001);
b0=s./(T0.*R);
[bop,nn,nnn,mm,mmm,kk]=optimal_b(Lat,C,D,R,s);


%path creation

for i=1:length(C)
    node(i).index=i;
    node(i).energy=node_energy(i);
    node(i).b=10;
    node(i).b0=b0(i);
    node(i).bop=bop(i);
    node(i).C=C(i);
    node(i).D=D(i);
    node(i).T=T0(i);
    
    %energy constraints taking into account battery level
    [node(i).omega,node(i).gamma]=energyconstraints(node(i).energy,max_energy);
end



%PUT IN ORDER NODES IN THE PATH
%from lower energies to higher ones

for i=2:(length(node))
    for j=1:(length(node)-1)
        if node(j).energy > node(j+1).energy
            aux = node(j); 
            node(j)= node(j+1);
            node(j+1)= aux;
        end
    end
end



%PERFORM BALANCING IN ONE PATH

avaiable_nodes=length(node);
groups_balanced=0;
not_balanced=[];
adjust=0;
j=1;

%++++++++++++++++++++++++beguin of for+++++++++++++++++++++++++++++++++++++
for k=1:length(node)
disp('####################################################################')
disp('####################################################################')
    
    disp('starting a new balancing group of nodes...')
    
    flag=0;
    selected_nodes=[];
    selected_C=node(k).C;
    selected_D=node(k).D;
    selected_nodes=[selected_nodes k];
    avaiable_nodes=avaiable_nodes-1;   
    
    %when there's 1 node alone:
    if avaiable_nodes == 0
        aux=0;
        for i=1:length(node)
            aux=aux+node(i).T;
        end
        aux=aux-node(k).T;
        node(k).T=Lat-aux; %max tx time allowed for last node unpaired
            
        node(k).b=ceil(s/((node(k).T)*R)); %round toward inf to satisfy lat
        node(k).T=s/((node(k).b)*R);
        
        aux=(((node(k).C)*((2.^(node(k).b))-1)+(node(k).D)).*s./(node(k).b));
        node(k).energy=(node(k).energy)-aux;
        node(k).Eused=aux;
        node(k).ratio=aux/E0;
        
        %updating omega and gamma
        [node(k).omega,node(k).gamma]=energyconstraints(node(k).energy,max_energy);
        
        not_balanced=[not_balanced k];

        break %breaking for
        
    elseif avaiable_nodes < 0
        break %breaking for
    end
    %end of alone node    
    
    
%**************************beguin of%while****************************
%*********************************************************************
%this is the block that adds a new node to the group of nodes that are
%trying to save power in the target node. 

    while flag==0
    disp('--------------------------------------------------------------')
    
    
    %BLOCKING NODE problem: only allowed to compute together less or equal
    %than 5 nodes (target+ 4 balance nodes)
        if length(selected_nodes) == max_group
            for i=1:length(selected_nodes)
                
                node(selected_nodes(i)).b=b(i);
                node(selected_nodes(i)).T=Topt(i);
                node(selected_nodes(i)).energy=node(selected_nodes(i)).energy-Eopt(i);
                node(selected_nodes(i)).Eused=Eopt(i);
                node(selected_nodes(i)).ratio=Eopt(i)/E0;
                
                %updating omega and gamma
                [node(selected_nodes(i)).omega,node(selected_nodes(i)).gamma]=energyconstraints(node(selected_nodes(i)).energy,max_energy);
                                                
                fprintf('node:');
                fprintf('  %d  ',selected_nodes(i));
                fprintf('not balanced due to max number of nodes in balance group was overcame\n');
            end
            
            j=j-1; %due to we come from GAMMA ERROR & ERROR1 and they add 1 to j in order
                   %to add a new node to the group. So we have to decrease in 1 unit j
            not_balanced=[not_balanced selected_nodes]; 
            break %breaking while
        end

        
        
        %here only goes in when more than 1 balancing node is required but
        %we dont have more than 1 balancing node avaiable. Ex: we have 2
        %nodes left, 1 is the target, the other the balancing node. So if
        %we cant perform balancing we are going to need another one but
        %there arent more avaiable. So both nodes are updated here.
        if avaiable_nodes ==0
            for i=1:length(selected_nodes)
                
                node(selected_nodes(i)).b=b(i);
                node(selected_nodes(i)).T=Topt(i);
                node(selected_nodes(i)).energy=node(selected_nodes(i)).energy-Eopt(i);
                node(selected_nodes(i)).Eused=Eopt(i);
                node(selected_nodes(i)).ratio=Eopt(i)/E0;
                
                %updating omega and gamma
                [node(selected_nodes(i)).omega,node(selected_nodes(i)).gamma]=energyconstraints(node(selected_nodes(i)).energy,max_energy);
                                                
                fprintf('node:');
                fprintf('  %d  ',selected_nodes(i));
                fprintf('not balanced \n');
            end
            
            not_balanced=[not_balanced selected_nodes]; 
            break %breaking while
        end
        
        
        
        
        %selection of one node more
        selected_C=[selected_C node(length(node)+2-k-j+adjust).C];
        selected_D=[selected_D node(length(node)+2-k-j+adjust).D];
        selected_nodes=[selected_nodes (length(node)+2-k-j+adjust)];
        avaiable_nodes=avaiable_nodes-1;

        
        
        
        %#################################################################
        %####################PERFORM THE BALANCE##########################
        tot_Lat=0;
        for i=1:length(selected_nodes)
            tot_Lat=tot_Lat+node(selected_nodes(i)).T;
        end  
        %++++++++++++++++IMPORTANT+++++++++++++++++++++++++++++++++++++++++
        
        %Now the order of the power parameters parameters is:
        %C=[C(target node) C(group node1) C(group node2) ...]
        %Where the associated battery levels are
        %Lgroupnode1 > Lgroupnode2 > ... > Ltargetnode
        
        %in order to execute easier the following next two functions we have
        %to organize the parameters C and D. So in C and D, the lower the index 
        %the higher the battery level. 
        %For instance: C(1) must be a power parameter of a node with 90% battery
        %              C(7) must be a power parameter of a node with 15% battery
        %              C(length) must be pow param of target node (10% of battery)
        %Always C(length(C)) will be the power parameter of the TARGET NODE
               
        aux=selected_C(1);
        selected_C(1:(length(selected_C)-1))=selected_C(2:length(selected_C));
        selected_C(length(selected_C))=aux;
        
        aux=selected_D(1);
        selected_D(1:(length(selected_D)-1))=selected_D(2:length(selected_D));
        selected_D(length(selected_D))=aux;
        
        %++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        
        
        [b,Eopt,notusedb0,notusedE0,notusedE_S,Topt]=optimal_b(tot_Lat,selected_C,selected_D,R,s);
        E_S=Eopt./E0;
        %selected_nodes
        %b
        
        [bb,energy_spent,Ebalanced,Tn]=balance_b(b,E0,E_S,node(selected_nodes(1)).omega,selected_C,selected_D,tot_Lat,R,s);
        %energy_spent
        %bb
        
        %balance_b can give 2 errors:
        %ERROR1:when the overspending is so hight: energy_spent = Inf
        %ERROR2:when omega can't be achieved: energy_spent = -1
        
        
        %++++++++++++++++IMPORTANT+++++++++++++++++++++++++++++++++++++++++
        %Now we have to reorder again the outputs of the balance_b.m
        %From [balancenode1 balancenode2 ...targetnode]
        %to [targetnode balancenode1 balancenode2 ....]
        aux=b(length(b));
        b(2:length(b))=b(1:(length(b)-1));
        b(1)=aux;
        
        aux=Eopt(length(Eopt));
        Eopt(2:length(Eopt))=Eopt(1:(length(Eopt)-1));
        Eopt(1)=aux;
        
        aux=Topt(length(Topt));
        Topt(2:length(Topt))=Topt(1:(length(Topt)-1));
        Topt(1)=aux;
        
        aux=bb(length(bb));
        bb(2:length(bb))=bb(1:(length(bb)-1));
        bb(1)=aux;
        
        aux=energy_spent(length(energy_spent));
        energy_spent(2:length(energy_spent))=energy_spent(1:(length(energy_spent)-1));
        energy_spent(1)=aux;
        
        aux=Ebalanced(length(Ebalanced));
        Ebalanced(2:length(Ebalanced))=Ebalanced(1:(length(Ebalanced)-1));
        Ebalanced(1)=aux;
        
        aux=Tn(length(Tn));
        Tn(2:length(Tn))=Tn(1:(length(Tn)-1));
        Tn(1)=aux;
        
        aux=selected_C(length(selected_C));
        selected_C(2:length(selected_C))=selected_C(1:(length(selected_C)-1));
        selected_C(1)=aux;
        
        aux=selected_D(length(selected_D));
        selected_D(2:length(selected_D))=selected_D(1:(length(selected_D)-1));
        selected_D(1)=aux;
        
        %#################################################################
        %####################END OF BALANCE###############################
        
        
        
        %#####################HANDLE ERRORS###############################
        
        %ERROR2
        %We have to free the node that is associated to the 1st one and
        %also we have to update the 1st node to the optimal case
        if energy_spent==-1
            disp('cant reach omega savings')
            %for i=1:length(selected_nodes)
                i=1;    
                node(selected_nodes(i)).b=ceil(node(selected_nodes(i)).b0);
                
                Topt2=s./((node(selected_nodes(i)).b).*R);
                node(selected_nodes(i)).T=Topt2;
                
                node(selected_nodes(i)).Eused = energy_model2(node(selected_nodes(i)).C,node(selected_nodes(i)).D,R,s,node(selected_nodes(i)).b);
                node(selected_nodes(i)).energy=node(selected_nodes(i)).energy-node(selected_nodes(i)).Eused;
                
                node(selected_nodes(i)).ratio=node(selected_nodes(i)).Eused/E0;
                
                adjust=adjust+1;
                avaiable_nodes=avaiable_nodes+1;
                
                [node(selected_nodes(i)).omega,node(selected_nodes(i)).gamma]=energyconstraints(node(selected_nodes(i)).energy,max_energy);
                
                not_balanced=[not_balanced selected_nodes(i)];
                
                fprintf('node:');
                fprintf('  %d  ',selected_nodes(i));
                fprintf('not balanced because unable to achieve omega= %d \n',node(selected_nodes(1)).omega);
            %end
            break %breaking while
        end
        
        
        
        %GAMMA ERROR & ERROR1
        %if we overcome the spending limit in some node we increase the number 
        %of nodes to help to satisfy the overspending constraint (<gamma)
        flag=1;
        disp('checking gamma...')
            
        for i=1:length(selected_nodes)
            if energy_spent(i)>node(selected_nodes(i)).gamma
                if i==1  %error1
                    fprintf('energy savings cant be reached with %d balancing nodes \n',(length(selected_nodes)-1));
                    disp('adding one node more to the balancing group...')
                    j=j+1;
                    flag=0;
                    break %breakin for
                else                
                    fprintf('over spending constraint violated in %d nd/rd/th node\n',selected_nodes(i));
                    disp('adding one node more to the balancing group...')
                    j=j+1;
                    flag=0;
                    break %breakin for
                end
            end
        end
        
        %#################END OF HANDLE ERRORS############################
        
        %if flag==1 means that every constraint are satisfied and the group 
        %of nodes are balanced so, it's the moment for updating the new values 
        if flag==1
            disp('all constraints satisfied: updating values...')
            groups_balanced=[groups_balanced selected_nodes 0];
            for i=1:length(selected_nodes)
                
                node(selected_nodes(i)).b=bb(i);
                node(selected_nodes(i)).T=Tn(i);
                node(selected_nodes(i)).energy=node(selected_nodes(i)).energy-Ebalanced(i);
                node(selected_nodes(i)).Eused=Ebalanced(i);
                node(selected_nodes(i)).ratio=Ebalanced(i)/E0;
                
                [node(selected_nodes(i)).omega,node(selected_nodes(i)).gamma]=energyconstraints(node(selected_nodes(i)).energy,max_energy);

                %disp('selected nodes:')
                %fprintf('  %d  ',selected_nodes(i));
                %fprintf('\n');
            end
        end
    end
%******************************************************************************    
%********************************end of while**********************************    
   
    
end
%++++++++++++++++++++++++end of for+++++++++++++++++++++++++++++++++++++

%end of main

%STATISTICS
time=0;
for i=1:length(node)
    ratio(i)=node(i).ratio*100;
    b(i)=node(i).b;
    b0(i)=node(i).b0;
    bop(i)=node(i).bop;
    time=time+node(i).T;
end
time=time

x=linspace(1,length(node),length(node));
subplot(1,2,1);
plot(x,b,'bo',x,b,'b',x,b0,'ro',x,b0,'r');
xlabel('node number(higher numbers are nodes with more remaining energy)')
ylabel('modulation parameter')
title('plot of balanced modulation parameters and theoretical optimal ones')
subplot(1,2,2);
plot(x,ratio,'bo',x,ratio,'b');
xlabel('node number(higher numbers are nodes with more remaining energy)')
ylabel('% energy spent')
title('plot of the % of energy spent regarding to the optimal energy')


 

%PUT IN ORDER THE NODES AGAIN FROM
%LOWER INDEX TO HIGHER ONE

for i=2:(length(node))
    for j=1:(length(node)-1)
        if node(j).index > node(j+1).index
            aux = node(j); 
            node(j)= node(j+1);
            node(j+1)= aux;
        end
    end
end
 


%end of main