function [b,Eopt,b0,E0,E_S,Tn]=optimal_b(T,C,D,R,s)

%IMPORTANT: In C and D, the lower the index the higher the battery level
%For instance: C(1) is a power parameter of a node with 90% battery
%              C(7) is a power parameter of a node with 15% battery
%********************INPUTS******************************************
%C is a vector(row) that symbolizes the tx power
%D is a vector(row) that symbolizes the electronic power
%R is the transmission rate
%s is the packet size
%T total path latency
%***********************OUTPUTS**************************************
%b (vector) is the optimal modulation parameter ceiled
%Eopt (vector) are the optimal energies for nodes with ceiling
%E_S is the energy ratio spending for nodes with rounded b's respect the theoric optimal
%b0 (vector) is the optimal parameter before ceiling
%E0 (vector)are the optimal energies before ceiling
%Tn are the latencies asociated to b's
%********************************************************************

%COMPUTE OPTIMAL CONSUMPTION
%disp('####################################################################')
%disp('####################################################################')

%Theoretical
disp('computing theoretical optimal energy...')
[E0,Tn]=optimal_energy(T,length(C),C,D,R,s,0.001);





%disp('computing ceiling...')
%Realistic(ceiling function)
b0=s./(Tn.*R);
b=round(b0);
Tn=s./(b.*R);
Ts=sum(Tn);




%the aim is to round the modulation parameters
%in order to achieve lower increments of b in nodes
%with less remaining energy. 

%j=length(C); wrong
j=1;
nodeflag=zeros(1,length(C));
while Ts > (T*(0.001+1))
    if nodeflag(j)==0
        if b(j)<b0(j)
%            fprintf('increasing modulation parameter of node %d \n',j)
            b(j)=b(j)+1;
        else
            nodeflag(j)=-1;
        end
    else
        b(j)=b(j)+1;
    end
    
    
    Tn=s./(b.*R);
    Ts=sum(Tn);
    
    %j=j-1; wrong
    %if j==0
    %    j=length(C);
    %end
    
    j=j+1;
    if j==(length(C)+1)
        j=1;
    end
    
end

for i = 1:length(C)
    Eopt(i)=((C(i).*((2.^b(i))-1)+(D(i))).*s./b(i));
end

%disp('ceiling finished')

E_S=Eopt./E0;


%for i = 1:length(C)
%    fprintf('percentage of energy spent in node %d: respect the theoric optimal is %f  \n',i,(E_S(i).*100))
%end

%disp('####################################################################')
%disp('####################################################################')


%end of optimal_b