function [node,n_tx,n_tx2,n_tx3,n_tx4,node_energy,node_energy2,node_energy3,node_energy4]=sim_distr(C,D,T,R,s,node_energy,level,distr)

%Simulator for tree topologies or other kinds. 
%INPUTS------------------------------
%C is a vector(row) that symbolizes the tx power
%D is a vector(row) that symbolizes the electronic power
%R is the transmission rate
%s is the packet size
%T total path latency
%node_energy is the energy of each node 

%The consumption factor of each node is given by the parameter distr.
%Parameter level determines at what energy level we want to start to 
%apply our protocol.

%OUTPUTS----------------------------
%n_tx is the n of transmission for our protocol
%n_tx2 the transmissions for the Non-Power-aware
%n_tx3 the transmissions for the Power-aware
%n_tx4 the transmissions performed with the power-aware when a certain
%level lower than the max battery is specified
%------------------------------------
%rate determines in our algorithm the up-date frequency
%of the modulation parameters of the whole path
b=0;
rate=100;
n_tx=0;
n_tx2=0;
n_tx3=0;
n_tx4=0;
node_energy2=node_energy;
node_energy3=node_energy;
node_energy4=node_energy;
coef=distr;
%#########################NON POWER AWARE PROTOCOL####################
a=1;
Emax=energy_model2(C,D,R,s,10);

while a >= 0    
        n_tx2=n_tx2+1;
        
        for i=1:length(C)
            node_energy2(i)=node_energy2(i)-(coef(i).*Emax(i));
        end

       
        for i=1:length(C)
            if node_energy2(i) <= 0
                fprintf('NON-AWARE protocol:number of total tx was %d \n',n_tx2);
                a=-1;
            end
        end
end


%#########################POWER AWARE PROTOCOL #############
a=1;
bop=optimal_b(T,C,D,R,s);
Eop=energy_model2(C,D,R,s,bop);

while a >= 0
    n_tx3=n_tx3+1;
    
    for i=1:length(C)
        node_energy3(i)=node_energy3(i)-(coef(i).*Eop(i));
    end


    for i=1:length(C)
        if node_energy3(i) <= 0
            fprintf('AWARE protocol:number of total tx was %d \n',n_tx3);
            a=-1;
        end
    end
end

%#########################POWER AWARE PROTOCOL (BALANCING)#############
a=1;
bop=optimal_b(T,C,D,R,s);
Eop=energy_model2(C,D,R,s,bop);

while a >= 0
    n_tx4=n_tx4+1;
    
    for i=1:length(C)
        node_energy4(i)=node_energy4(i)-(coef(i).*Eop(i));
    end
    

    for i=1:length(C)
        if node_energy4(i) <= level
            fprintf('changing to balancing mode');
            a=-1;
        end
    end
end

for i=1:length(C)
    node_energy(i)=node_energy4(i);
end

a=1;
n_tx=n_tx4;
while a >= 0
    
    %main also computes the tx of 1 packet over all the path
    [node,groups_balanced,not_balanced]=main(C,D,T,R,s,node_energy);
    n_tx=n_tx+1;

    for i=1:length(node)
        node_energy(i)=node(i).energy;
        C(i)=node(i).C;
        D(i)=node(i).D;
        b(i)=node(i).b;
    end
    
    E=energy_model2(C,D,R,s,b);
    
    for i=1:length(C)
        node_energy(i)=node_energy(i)-((coef(i)-1).*E(i));
        node(i).energy=node_energy(i); 
    end
    
   
    for i=1:length(node)
               
        if node_energy(i) <= 0
            fprintf('node %d is down \n',node(i).index);
            fprintf('number of total tx was %d \n',n_tx);
            a=-1;
        end
    end
    
    if a==-1
         return
    end
 
    
    
    
    for i=1:rate
        
        for i=1:length(C)
            node_energy(i)=node_energy(i)-(coef(i).*E(i));
            node(i).energy=node_energy(i);
    
        end
       
        n_tx=n_tx+1;

        
        for i=1:length(node)
            if node_energy(i) <= 0
                fprintf('node %d is down \n',node(i).index);
                fprintf('number of total tx was %d \n',n_tx);
                a=-1;
            end
        end
        
        if a==-1
            return
        end
    end    
end
